package com.sharix.sportsmanfriend.dialogs;

import android.Manifest;
import android.app.Dialog;
import android.content.pm.PackageManager;
import android.graphics.Bitmap;
import android.graphics.Canvas;
import android.os.Bundle;
import android.view.LayoutInflater;
import android.widget.ImageButton;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.core.app.ActivityCompat;

import com.google.android.gms.location.FusedLocationProviderClient;
import com.google.android.gms.location.LocationServices;
import com.sharix.sportsmanfriend.BaseDialogClass;
import com.sharix.sportsmanfriend.MapHelperClass;
import com.sharix.sportsmanfriend.R;

import org.osmdroid.views.MapView;

import java.io.ByteArrayOutputStream;

public class SelectLocationDialog extends BaseDialogClass {
    private final int PERMISSIONS_REQUEST_ACCESS_FINE_LOCATION = 2;
    private ImageButton btnZoomIn, btnZoomOut, btnMyLocation;
    private MapView map = null;
    //определяет последнюю известную локацию
    private FusedLocationProviderClient fusedLocationProviderClient;

    @NonNull
    @Override
    public Dialog onCreateDialog(@Nullable Bundle savedInstanceState) {
        LayoutInflater inflater = requireActivity().getLayoutInflater();
        view = inflater.inflate(R.layout.dialog_select_location, null);

        map = view.findViewById(R.id.dialog_map);
        btnZoomIn = view.findViewById(R.id.btn_zoomIn);
        btnZoomOut = view.findViewById(R.id.btn_zoomOut);
        btnMyLocation = view.findViewById(R.id.btnMyLocation);

        fusedLocationProviderClient = LocationServices.getFusedLocationProviderClient(requireActivity());

        MapHelperClass.configureMap(requireActivity() ,map,55.7522200, 37.6155600, 15);

        addClickEventToAllViews();

        return super.onCreateDialog(savedInstanceState);
    }

    @Override
    protected Bundle DataToTransfer() {
        Bundle bundle = new Bundle();
        bundle.putString("FromBlock", "LocationBlock");
        bundle.putDouble("latitude", MapHelperClass.GetMapCenter(map).getLatitude());
        bundle.putDouble("longitude", MapHelperClass.GetMapCenter(map).getLongitude());
        bundle.putByteArray("screenshot", getScreenshot());
        return bundle;
    }

    @Override
    protected void addClickEventToAllViews() {
        super.addClickEventToAllViews();

        btnZoomIn.setOnClickListener(v -> MapHelperClass.ZoomIn(map));

        btnZoomOut.setOnClickListener(v -> MapHelperClass.ZoomOut(map));

        btnMyLocation.setOnClickListener(v -> {
            if (ActivityCompat.checkSelfPermission(requireActivity(), Manifest.permission.ACCESS_FINE_LOCATION) != PackageManager.PERMISSION_GRANTED
                    && ActivityCompat.checkSelfPermission(requireActivity(), Manifest.permission.ACCESS_COARSE_LOCATION) != PackageManager.PERMISSION_GRANTED) {
                String[] permission = {Manifest.permission.ACCESS_FINE_LOCATION, Manifest.permission.ACCESS_COARSE_LOCATION};
                requestPermissions(permission, PERMISSIONS_REQUEST_ACCESS_FINE_LOCATION);
            }

            fusedLocationProviderClient.getLastLocation()
                .addOnSuccessListener(requireActivity(), location -> {
                    if (location != null) {
                        MapHelperClass.SetCenterOfMap(map, location.getLatitude(), location.getLongitude());
                        MapHelperClass.SetZoomOfMap(map, 18);
                    }
                });

        });
    }

    @Override
    public void onRequestPermissionsResult(int requestCode, @NonNull String[] permissions, @NonNull int[] grantResults) {
        super.onRequestPermissionsResult(requestCode, permissions, grantResults); {
            if (grantResults.length > 0 && grantResults[0] == PackageManager.PERMISSION_GRANTED) {
                requireActivity().recreate();
            }
        }
    }

    private byte[] getScreenshot(){

        float density = getResources().getDisplayMetrics().density;
        int pixels = (int) (100 * density);

        Bitmap screenshot = Bitmap.createBitmap(pixels, pixels, Bitmap.Config.ARGB_8888);
        Canvas canvas = new Canvas(screenshot);
        int centerX = view.getWidth() / 2;
        int centerY = view.getHeight() / 2;
        canvas.translate(-centerX + pixels / 2, -centerY + pixels / 2);
        view.draw(canvas);
        ByteArrayOutputStream baos = new ByteArrayOutputStream();
        screenshot.compress(Bitmap.CompressFormat.PNG, 100, baos);
        return baos.toByteArray();
    }
}
